﻿using jp.co.systembase.json;
using jp.co.systembase.report;
using jp.co.systembase.report.data;
using jp.co.systembase.report.renderer.gdi;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace example
{
    internal static class Program
    {

        [STAThread]
        static void Main()
        {
            var report = new Report(Json.Read("example.rrpt"));

            // 明細１/２のデータを各グループに割当
            var data1 = new PagedDataSource(getDataTable1(), 5);
            var data2 = new PagedDataSource(getDataTable2(), 10);
            var dataProvider = new GroupDataProvider();
            dataProvider.GroupDataMap.Add("group_data1", data1);
            dataProvider.GroupDataMap.Add("group_data2", data2);

            // 必要なページ数を表すデータを割当
            report.Fill(new PageCountDataSource(data1, data2), dataProvider);
            var pages = report.GetPages();

            var preview = new FmPrintPreview(new Printer(pages));
            preview.ShowDialog();
        }

        // 明細１のデータ
        private static DataTable getDataTable1()
        {
            DataTable ret = new DataTable();
            ret.Columns.Add("col1", typeof(Decimal));
            for (var i = 0; i < 4; i++)
            {
                ret.Rows.Add(i);
            }
            return ret;
        }

        // 明細２のデータ
        private static DataTable getDataTable2()
        {
            DataTable ret = new DataTable();
            ret.Columns.Add("col2", typeof(Decimal));
            for (var i = 0; i < 12; i++)
            {
                ret.Rows.Add(i);
            }
            return ret;
        }

        // 明細データをページ分割
        private class PagedDataSource : ReportDataSource
        {
            private int PageSize;
            public PagedDataSource(DataTable dataTable, int pageSize) : base(dataTable)
            {
                PageSize = pageSize;
            }

            public override object Get(int i, string key)
            {
                if (key == "page_index")
                {
                    return i / PageSize;
                }
                else
                {
                    return base.Get(i, key);
                }
            }

            public int PageCount()
            {
                return (Size() + PageSize - 1) / PageSize;
            }

        }

        // 2つの明細データを受け取って必要なページ数を返す
        private class PageCountDataSource : IReportDataSource
        {
            private int PageCount;
            public PageCountDataSource(PagedDataSource data1, PagedDataSource data2)
            {
                PageCount = Math.Max(data1.PageCount(), data2.PageCount());
            }

            public object Get(int i, string key)
            {
                if (key == "page_index")
                {
                    return i;
                }
                else
                {
                    return null;
                }
            }

            public int Size()
            {
                return PageCount;
            }
        }


    }
}
